@@ -1,3 +1,8 @@
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
+#
+
 """Deal with representations of Markov Models.
 """
 # standard modules
@@ -5,10 +10,78 @@ import copy
 import math
 import random
 
-# biopython
+# TODO - Take advantage of defaultdict once Python 2.4 is dead?
+# from collections import defaultdict
+
+from Bio._py3k import range
+
 from Bio.Seq import MutableSeq
 
-class MarkovModelBuilder:
+
+def _gen_random_array(n):
+    """ Return an array of n random numbers, where the elements of the array sum
+    to 1.0"""
+    randArray = [random.random() for i in range(n)]
+    total = sum(randArray)
+    normalizedRandArray = [x/total for x in randArray]
+
+    return normalizedRandArray
+
+
+def _calculate_emissions(emission_probs):
+    """Calculate which symbols can be emitted in each state
+    """
+    # loop over all of the state-symbol duples, mapping states to
+    # lists of emitted symbols
+    emissions = dict()
+    for state, symbol in emission_probs:
+        try:
+            emissions[state].append(symbol)
+        except KeyError:
+            emissions[state] = [symbol]
+
+    return emissions
+
+
+def _calculate_from_transitions(trans_probs):
+    """Calculate which 'from transitions' are allowed for each state
+
+    This looks through all of the trans_probs, and uses this dictionary
+    to determine allowed transitions. It converts this information into
+    a dictionary, whose keys are source states and whose values are
+    lists of destination states reachable from the source state via a
+    transition.
+    """
+    transitions = dict()
+    for from_state, to_state in trans_probs:
+        try:
+            transitions[from_state].append(to_state)
+        except KeyError:
+            transitions[from_state] = [to_state]
+
+    return transitions
+
+
+def _calculate_to_transitions(trans_probs):
+    """Calculate which 'to transitions' are allowed for each state
+
+    This looks through all of the trans_probs, and uses this dictionary
+    to determine allowed transitions. It converts this information into
+    a dictionary, whose keys are destination states and whose values are
+    lists of source states from which the destination is reachable via a
+    transition.
+    """
+    transitions = dict()
+    for from_state, to_state in trans_probs:
+        try:
+            transitions[to_state].append(from_state)
+        except KeyError:
+            transitions[to_state] = [from_state]
+
+    return transitions
+
+
+class MarkovModelBuilder(object):
     """Interface to build up a Markov Model.
 
     This class is designed to try to separate the task of specifying the
@@ -20,7 +93,7 @@ class MarkovModelBuilder:
     """
     # the default pseudo counts to use
     DEFAULT_PSEUDO = 1
-    
+
     def __init__(self, state_alphabet, emission_alphabet):
         """Initialize a builder to create Markov Models.
 
@@ -28,13 +101,17 @@ class MarkovModelBuilder:
 
         o state_alphabet -- An alphabet containing all of the letters that
         can appear in the states
-       
+
         o emission_alphabet -- An alphabet containing all of the letters for
         states that can be emitted by the HMM.
         """
         self._state_alphabet = state_alphabet
         self._emission_alphabet = emission_alphabet
-        
+
+        # probabilities for the initial state, initialized by calling
+        # set_initial_probabilities (required)
+        self.initial_prob = {}
+
         # the probabilities for transitions and emissions
         # by default we have no transitions and all possible emissions
         self.transition_prob = {}
@@ -75,26 +152,75 @@ class MarkovModelBuilder:
                 all_counts[(first_state, second_state)] = self.DEFAULT_PSEUDO
 
         return all_counts
-                
+
     def get_markov_model(self):
         """Return the markov model corresponding with the current parameters.
 
         Each markov model returned by a call to this function is unique
         (ie. they don't influence each other).
         """
+
+        # user must set initial probabilities
+        if not self.initial_prob:
+            raise Exception("set_initial_probabilities must be called to " +
+                            "fully initialize the Markov model")
+
+        initial_prob = copy.deepcopy(self.initial_prob)
         transition_prob = copy.deepcopy(self.transition_prob)
         emission_prob = copy.deepcopy(self.emission_prob)
         transition_pseudo = copy.deepcopy(self.transition_pseudo)
         emission_pseudo = copy.deepcopy(self.emission_pseudo)
-        
-        return HiddenMarkovModel(transition_prob, emission_prob,
+
+        return HiddenMarkovModel(initial_prob, transition_prob, emission_prob,
                                  transition_pseudo, emission_pseudo)
 
+    def set_initial_probabilities(self, initial_prob):
+        """Set initial state probabilities.
+
+        initial_prob is a dictionary mapping states to probabilities.
+        Suppose, for example, that the state alphabet is ['A', 'B']. Call
+        set_initial_prob({'A': 1}) to guarantee that the initial
+        state will be 'A'. Call set_initial_prob({'A': 0.5, 'B': 0.5})
+        to make each initial state equally probable.
+
+        This method must now be called in order to use the Markov model
+        because the calculation of initial probabilities has changed
+        incompatibly; the previous calculation was incorrect.
+
+        If initial probabilities are set for all states, then they should add up
+        to 1. Otherwise the sum should be <= 1. The residual probability is
+        divided up evenly between all the states for which the initial
+        probability has not been set. For example, calling
+        set_initial_prob({}) results in P('A') = 0.5 and P('B') = 0.5,
+        for the above example.
+        """
+        self.initial_prob = copy.copy(initial_prob)
+
+        # ensure that all referenced states are valid
+        for state in initial_prob:
+            assert state in self._state_alphabet.letters, \
+                   "State %s was not found in the sequence alphabet" % state
+
+        # distribute the residual probability, if any
+        num_states_not_set =\
+            len(self._state_alphabet.letters) - len(self.initial_prob)
+        if num_states_not_set < 0:
+            raise Exception("Initial probabilities can't exceed # of states")
+        prob_sum = sum(self.initial_prob.values())
+        if prob_sum > 1.0:
+            raise Exception("Total initial probability cannot exceed 1.0")
+        if num_states_not_set > 0:
+            prob = (1.0 - prob_sum) / num_states_not_set
+            for state in self._state_alphabet.letters:
+                if state not in self.initial_prob:
+                    self.initial_prob[state] = prob
+
     def set_equal_probabilities(self):
         """Reset all probabilities to be an average value.
 
-        This resets the values of all allowed transitions and all allowed
-        emissions to be equal to 1 divided by the number of possible elements.
+        Resets the values of all initial probabilities and all allowed
+        transitions and all allowed emissions to be equal to 1 divided by the
+        number of possible elements.
 
         This is useful if you just want to initialize a Markov Model to
         starting values (ie. if you have no prior notions of what the
@@ -107,34 +233,77 @@ class MarkovModelBuilder:
         emissions to total up to 1, so it doesn't ensure that the sum of
         each set of transitions adds up to 1.
         """
-        # first set the transitions
-        new_trans_prob = float(1) / float(len(self.transition_prob.keys()))
-        for key in self.transition_prob.keys():
+
+        # set initial state probabilities
+        new_initial_prob = float(1) / float(len(self.transition_prob))
+        for state in self._state_alphabet.letters:
+            self.initial_prob[state] = new_initial_prob
+
+        # set the transitions
+        new_trans_prob = float(1) / float(len(self.transition_prob))
+        for key in self.transition_prob:
             self.transition_prob[key] = new_trans_prob
 
-        # now set the emissions
-        new_emission_prob = float(1) / float(len(self.emission_prob.keys()))
-        for key in self.emission_prob.keys():
+        # set the emissions
+        new_emission_prob = float(1) / float(len(self.emission_prob))
+        for key in self.emission_prob:
             self.emission_prob[key] = new_emission_prob
-            
+
+    def set_random_initial_probabilities(self):
+        """Set all initial state probabilities to a randomly generated distribution.
+        Returns the dictionary containing the initial probabilities.
+        """
+        initial_freqs = _gen_random_array(len(self._state_alphabet.letters))
+        for state in self._state_alphabet.letters:
+            self.initial_prob[state] = initial_freqs.pop()
+
+        return self.initial_prob
+
+    def set_random_transition_probabilities(self):
+        """Set all allowed transition probabilities to a randomly generated distribution.
+        Returns the dictionary containing the transition probabilities.
+        """
+
+        if not self.transition_prob:
+            raise Exception("No transitions have been allowed yet. " +
+                            "Allow some or all transitions by calling " +
+                            "allow_transition or allow_all_transitions first.")
+
+        transitions_from = _calculate_from_transitions(self.transition_prob)
+        for from_state in transitions_from:
+            freqs = _gen_random_array(len(transitions_from[from_state]))
+            for to_state in transitions_from[from_state]:
+                self.transition_prob[(from_state, to_state)] = freqs.pop()
+
+        return self.transition_prob
+
+    def set_random_emission_probabilities(self):
+        """Set all allowed emission probabilities to a randomly generated
+        distribution.  Returns the dictionary containing the emission
+        probabilities.
+        """
+
+        if not self.emission_prob:
+            raise Exception("No emissions have been allowed yet. " +
+                            "Allow some or all emissions.")
+
+        emissions = _calculate_emissions(self.emission_prob)
+        for state in emissions:
+            freqs = _gen_random_array(len(emissions[state]))
+            for symbol in emissions[state]:
+                self.emission_prob[(state, symbol)] = freqs.pop()
+
+        return self.emission_prob
 
     def set_random_probabilities(self):
         """Set all probabilities to randomly generated numbers.
 
-        This will reset the value of all allowed transitions and emissions
-        to random values.
-
-        Warning 1 -- This will reset any currently set probabibilities.
-
-        Warning 2 -- This does not check to ensure that the sum of
-        all of the probabilities is less then 1. It just randomly assigns
-        a probability to each
+        Resets probabilities of all initial states, transitions, and
+        emissions to random values.
         """
-        for key in self.transition_prob.keys():
-            self.transition_prob[key] = random.random()
-
-        for key in self.emission_prob.keys():
-            self.emission_prob[key] = random.random()
+        self.set_random_initial_probabilities()
+        self.set_random_transition_probabilities()
+        self.set_random_emission_probabilities()
 
     # --- functions to deal with the transitions in the sequence
 
@@ -154,18 +323,18 @@ class MarkovModelBuilder:
 
         # now set any probabilities and pseudo counts that
         # were previously set
-        for set_key in self.transition_prob.keys():
+        for set_key in self.transition_prob:
             all_probs[set_key] = self.transition_prob[set_key]
 
-        for set_key in self.transition_pseudo.keys():
+        for set_key in self.transition_pseudo:
             all_pseudo[set_key] = self.transition_pseudo[set_key]
 
         # finally reinitialize the transition probs and pseudo counts
         self.transition_prob = all_probs
         self.transition_pseudo = all_pseudo
 
-    def allow_transition(self, from_state, to_state, probability = None,
-                         pseudocount = None):
+    def allow_transition(self, from_state, to_state, probability=None,
+                         pseudocount=None):
         """Set a transition as being possible between the two states.
 
         probability and pseudocount are optional arguments
@@ -178,12 +347,12 @@ class MarkovModelBuilder:
         """
         # check the sanity of adding these states
         for state in [from_state, to_state]:
-            assert state in self._state_alphabet, \
+            assert state in self._state_alphabet.letters, \
                    "State %s was not found in the sequence alphabet" % state
 
         # ensure that the states are not already set
-        if ((from_state, to_state) not in self.transition_prob.keys() and 
-            (from_state, to_state) not in self.transition_pseudo.keys()):
+        if (from_state, to_state) not in self.transition_prob and \
+           (from_state, to_state) not in self.transition_pseudo:
             # set the initial probability
             if probability is None:
                 probability = 0
@@ -192,9 +361,9 @@ class MarkovModelBuilder:
             # set the initial pseudocounts
             if pseudocount is None:
                 pseudcount = self.DEFAULT_PSEUDO
-            self.transition_pseudo[(from_state, to_state)] = pseudocount 
+            self.transition_pseudo[(from_state, to_state)] = pseudocount
         else:
-            raise KeyError("Transtion from %s to %s is already allowed."
+            raise KeyError("Transition from %s to %s is already allowed."
                            % (from_state, to_state))
 
     def destroy_transition(self, from_state, to_state):
@@ -216,7 +385,7 @@ class MarkovModelBuilder:
         Raises:
         KeyError if the transition is not allowed.
         """
-        if self.transition_prob.has_key((from_state, to_state)):
+        if (from_state, to_state) in self.transition_prob:
             self.transition_prob[(from_state, to_state)] = probability
         else:
             raise KeyError("Transition from %s to %s is not allowed."
@@ -234,7 +403,7 @@ class MarkovModelBuilder:
         Raises:
         KeyError if the transition is not allowed.
         """
-        if self.transition_pseudo.has_key((from_state, to_state)):
+        if (from_state, to_state) in self.transition_pseudo:
             self.transition_pseudo[(from_state, to_state)] = count
         else:
             raise KeyError("Transition from %s to %s is not allowed."
@@ -248,7 +417,7 @@ class MarkovModelBuilder:
         Raises:
         KeyError if the emission from the given state is not allowed.
         """
-        if self.emission_prob.has_key((seq_state, emission_state)):
+        if (seq_state, emission_state) in self.emission_prob:
             self.emission_prob[(seq_state, emission_state)] = probability
         else:
             raise KeyError("Emission of %s from %s is not allowed."
@@ -266,17 +435,18 @@ class MarkovModelBuilder:
         Raises:
         KeyError if the emission from the given state is not allowed.
         """
-        if self.emission_pseudo.has_key((seq_state, emission_state)):
+        if (seq_state, emission_state) in self.emission_pseudo:
             self.emission_pseudo[(seq_state, emission_state)] = count
         else:
             raise KeyError("Emission of %s from %s is not allowed."
                            % (emission_state, seq_state))
 
-class HiddenMarkovModel:
+
+class HiddenMarkovModel(object):
     """Represent a hidden markov model that can be used for state estimation.
     """
-    def __init__(self, transition_prob, emission_prob, transition_pseudo,
-                 emission_pseudo):
+    def __init__(self, initial_prob, transition_prob, emission_prob,
+                 transition_pseudo, emission_pseudo):
         """Initialize a Markov Model.
 
         Note: You should use the MarkovModelBuilder class instead of
@@ -284,52 +454,40 @@ class HiddenMarkovModel:
 
         Arguments:
 
+        o initial_prob - A dictionary of initial probabilities for all states.
+
         o transition_prob -- A dictionary of transition probabilities for all
         possible transitions in the sequence.
 
-        o emission_prob -- A dictionary of emissions probabilities for all
+        o emission_prob -- A dictionary of emission probabilities for all
         possible emissions from the sequence states.
 
         o transition_pseudo -- Pseudo-counts to be used for the transitions,
         when counting for purposes of estimating transition probabilities.
 
-        o emission_pseduo -- Pseudo-counts fo tbe used for the emissions,
+        o emission_pseudo -- Pseudo-counts to be used for the emissions,
         when counting for purposes of estimating emission probabilities.
         """
+
+        self.initial_prob = initial_prob
+
         self._transition_pseudo = transition_pseudo
         self._emission_pseudo = emission_pseudo
-        
+
         self.transition_prob = transition_prob
         self.emission_prob = emission_prob
 
-        # a dictionary of the possible transitions from one letter to the next
-        # the keys are the letter, and the values are lists of letters that
-        # can be transitioned to
+        # a dictionary of the possible transitions from each state
+        # each key is a source state, mapped to a list of the destination states
+        # that are reachable from the source state via a transition
         self._transitions_from = \
-           self._calculate_from_transitions(self.transition_prob)
+           _calculate_from_transitions(self.transition_prob)
 
-    def _calculate_from_transitions(self, trans_probs):
-        """Calculate which 'from transitions' are allowed for each letter.
-
-        This looks through all of the trans_probs, and uses this dictionary
-        to determine allowed transitions. It converts this information into
-        a dictionary, whose keys are the transition letters and whose
-        values are a list of allowed letters to transition to.
-        """
-        from_transitions = {}
-
-        # loop over all of the different transitions
-        for trans_key in trans_probs.keys():
-            # if the letter to 'transition from' already exists, add the
-            # new letter which can be 'transitioned to' to the list
-            try:
-                from_transitions[trans_key[0]].append(trans_key[1])
-            # otherwise create the list and add the letter
-            except KeyError:
-                from_transitions[trans_key[0]] = []
-                from_transitions[trans_key[0]].append(trans_key[1])
-
-        return from_transitions
+        # a dictionary of the possible transitions to each state
+        # each key is a destination state, mapped to a list of source states
+        # from which the destination is reachable via a transition
+        self._transitions_to = \
+           _calculate_to_transitions(self.transition_prob)
 
     def get_blank_transitions(self):
         """Get the default transitions for the model.
@@ -343,7 +501,7 @@ class HiddenMarkovModel:
 
     def get_blank_emissions(self):
         """Get the starting default emmissions for each sequence.
-        
+
         This returns a dictionary of the default emmissions for each
         letter. The dictionary is structured with keys as
         (seq_letter, emmission_letter) and values as the starting number
@@ -352,16 +510,30 @@ class HiddenMarkovModel:
         return self._emission_pseudo
 
     def transitions_from(self, state_letter):
-        """Get all transitions which can happen from the given state.
+        """Get all destination states to which there are transitions from the
+        state_letter source state.
 
-        This returns all letters which the given state_letter is allowed
-        to transition to. An empty list is returned if no letters are possible.
+        This returns all letters which the given state_letter can transition
+        to. An empty list is returned if state_letter has no outgoing
+        transitions.
         """
-        try:
+        if state_letter in self._transitions_from:
             return self._transitions_from[state_letter]
-        except KeyError:
+        else:
             return []
-        
+
+    def transitions_to(self, state_letter):
+        """Get all source states from which there are transitions to the
+        state_letter destination state.
+
+        This returns all letters which the given state_letter is reachable
+        from. An empty list is returned if state_letter is unreachable.
+        """
+        if state_letter in self._transitions_to:
+            return self._transitions_to[state_letter]
+        else:
+            return []
+
     def viterbi(self, sequence, state_alphabet):
         """Calculate the most probable state path using the Viterbi algorithm.
 
@@ -378,88 +550,91 @@ class HiddenMarkovModel:
         o state_alphabet -- The alphabet of the possible state sequences
         that can be generated.
         """
-        # calculate logarithms of the transition and emission probs
+
+        # calculate logarithms of the initial, transition, and emission probs
+        log_initial = self._log_transform(self.initial_prob)
         log_trans = self._log_transform(self.transition_prob)
         log_emission = self._log_transform(self.emission_prob)
 
         viterbi_probs = {}
         pred_state_seq = {}
         state_letters = state_alphabet.letters
-        # --- initialization
-        #
-        # NOTE: My index numbers are one less than what is given in Durbin
-        # et al, since we are indexing the sequence going from 0 to
-        # (Length - 1) not 1 to Length, like in Durbin et al.
-        #
-        # v_{0}(0) = 1
-        viterbi_probs[(state_letters[0], -1)] = 1
-        # v_{k}(0) = 0 for k > 0
-        for state_letter in state_letters[1:]:
-            viterbi_probs[(state_letter, -1)] = 0
 
         # --- recursion
         # loop over the training squence (i = 1 .. L)
+        # NOTE: My index numbers are one less than what is given in Durbin
+        # et al, since we are indexing the sequence going from 0 to
+        # (Length - 1) not 1 to Length, like in Durbin et al.
         for i in range(0, len(sequence)):
-            # now loop over all of the letters in the state path
-            for main_state in state_letters:
+            # loop over all of the possible i-th states in the state path
+            for cur_state in state_letters:
                 # e_{l}(x_{i})
-                emission_part = log_emission[(main_state, sequence[i])]
+                emission_part = log_emission[(cur_state, sequence[i])]
 
-                # loop over all possible states
-                possible_state_probs = {}
-                for cur_state in self.transitions_from(main_state):
-                    # a_{kl}
-                    trans_part = log_trans[(cur_state, main_state)]
+                max_prob = 0
+                if i == 0:
+                    # for the first state, use the initial probability rather
+                    # than looking back to previous states
+                    max_prob = log_initial[cur_state]
+                else:
+                    # loop over all possible (i-1)-th previous states
+                    possible_state_probs = {}
+                    for prev_state in self.transitions_to(cur_state):
+                        # a_{kl}
+                        trans_part = log_trans[(prev_state, cur_state)]
 
-                    # v_{k}(i - 1)
-                    viterbi_part = viterbi_probs[(cur_state, i - 1)]
-                    cur_prob = viterbi_part + trans_part
+                        # v_{k}(i - 1)
+                        viterbi_part = viterbi_probs[(prev_state, i - 1)]
+                        cur_prob = viterbi_part + trans_part
 
-                    possible_state_probs[cur_state] = cur_prob
+                        possible_state_probs[prev_state] = cur_prob
 
-                # finally calculate the viterbi probability using the max
-                max_prob = max(possible_state_probs.values())
-                viterbi_probs[(main_state, i)] = (emission_part + max_prob)
+                    # calculate the viterbi probability using the max
+                    max_prob = max(possible_state_probs.values())
+
+                # v_{k}(i)
+                viterbi_probs[(cur_state, i)] = (emission_part + max_prob)
+
+                if i > 0:
+                    # get the most likely prev_state leading to cur_state
+                    for state in possible_state_probs:
+                        if possible_state_probs[state] == max_prob:
+                            pred_state_seq[(i - 1, cur_state)] = state
+                            break
 
-                # now get the most likely state
-                for state in possible_state_probs.keys():
-                    if possible_state_probs[state] == max_prob:
-                        pred_state_seq[(i - 1, main_state)] = state
-                        break
-                    
         # --- termination
         # calculate the probability of the state path
-        # loop over all letters
+        # loop over all states
         all_probs = {}
         for state in state_letters:
             # v_{k}(L)
-            viterbi_part = viterbi_probs[(state, len(sequence) - 1)]
-            # a_{k0}
-            transition_part = log_trans[(state, state_letters[0])]
-
-            all_probs[state] = viterbi_part * transition_part
+            all_probs[state] = viterbi_probs[(state, len(sequence) - 1)]
 
         state_path_prob = max(all_probs.values())
 
         # find the last pointer we need to trace back from
         last_state = ''
-        for state in all_probs.keys():
+        for state in all_probs:
             if all_probs[state] == state_path_prob:
                 last_state = state
 
         assert last_state != '', "Didn't find the last state to trace from!"
-                
+
         # --- traceback
         traceback_seq = MutableSeq('', state_alphabet)
-        
-        loop_seq = range(0, len(sequence))
+
+        loop_seq = list(range(1, len(sequence)))
         loop_seq.reverse()
 
-        cur_state = last_state
+        # last_state is the last state in the most probable state sequence.
+        # Compute that sequence by walking backwards in time. From the i-th
+        # state in the sequence, find the (i-1)-th state as the most
+        # probable state preceding the i-th state.
+        state = last_state
+        traceback_seq.append(state)
         for i in loop_seq:
-            traceback_seq.append(cur_state)
-            
-            cur_state = pred_state_seq[(i - 1, cur_state)]
+            state = pred_state_seq[(i - 1, state)]
+            traceback_seq.append(state)
 
         # put the traceback sequence in the proper orientation
         traceback_seq.reverse()
@@ -469,16 +644,23 @@ class HiddenMarkovModel:
     def _log_transform(self, probability):
         """Return log transform of the given probability dictionary.
 
-        When calculating the Viterbi equation, we need to deal with things
-        as sums of logs instead of products of probabilities, so that we
-        don't get underflow errors.. This copies the given probability
-        dictionary and returns the same dictionary with everything
-        transformed with a log.
+        When calculating the Viterbi equation, add logs of probabilities rather
+        than multiplying probabilities, to avoid underflow errors. This method
+        returns a new dictionary with the same keys as the given dictionary
+        and log-transformed values.
         """
         log_prob = copy.copy(probability)
-
-        for key in log_prob.keys():
-            log_prob[key] = math.log(log_prob[key])
+        try:
+            neg_inf = float("-inf")
+        except ValueError:
+            # On Python 2.5 or older that was handled in C code,
+            # and failed on Windows XP 32bit
+            neg_inf = - 1E400
+        for key in log_prob:
+            prob = log_prob[key]
+            if prob > 0:
+                log_prob[key] = math.log(log_prob[key])
+            else:
+                log_prob[key] = neg_inf
 
         return log_prob
-    
