@@ -1,7 +1,12 @@
+# This code is part of the Biopython distribution and governed by its
+# license.  Please see the LICENSE file that should have been included
+# as part of this package.
+#
+
 """Deal with Motifs or Signatures allowing ambiguity in the sequences.
 
 This class contains Schema which deal with Motifs and Signatures at
-a higher level, by introducing `don't care` (ambiguity) symbols into
+a higher level, by introducing \`don't care\` (ambiguity) symbols into
 the sequences. For instance, you could combine the following Motifs:
 
 'GATC', 'GATG', 'GATG', 'GATT'
@@ -11,15 +16,18 @@ character can be anything. This helps us condense a whole ton of
 motifs or signatures.
 """
 # standard modules
+from __future__ import print_function
+
 import random
 import re
 
-# biopython
+from Bio._py3k import range
+
 from Bio import Alphabet
 from Bio.Seq import MutableSeq
 
 # neural network libraries
-from Pattern import PatternRepository
+from .Pattern import PatternRepository
 
 # genetic algorithm libraries
 from Bio.GA import Organism
@@ -30,7 +38,9 @@ from Bio.GA.Repair.Stabilizing import AmbiguousRepair
 from Bio.GA.Selection.Tournament import TournamentSelection
 from Bio.GA.Selection.Diversity import DiversitySelection
 
-class Schema:
+__docformat__ = "restructuredtext en"
+
+class Schema(object):
     """Deal with motifs that have ambiguity characters in it.
 
     This motif class allows specific ambiguity characters and tries to
@@ -43,10 +53,10 @@ class Schema:
         """Initialize with ambiguity information.
 
         Arguments:
-        
+
         o ambiguity_info - A dictionary which maps letters in the motifs to
         the ambiguous characters which they might represent. For example,
-        {'R' : 'AG'} specifies that Rs in the motif can match a A or a G.
+        {'R' : 'AG'} specifies that Rs in the motif can match an A or a G.
         All letters in the motif must be represented in the ambiguity_info
         dictionary.
         """
@@ -57,11 +67,11 @@ class Schema:
 
     def encode_motif(self, motif):
         """Encode the passed motif as a regular expression pattern object.
-        
+
         Arguments:
 
         o motif - The motif we want to encode. This should be a string.
-        
+
         Returns:
         A compiled regular expression pattern object that can be used
         for searching strings.
@@ -138,9 +148,7 @@ class Schema:
     def all_unambiguous(self):
         """Return a listing of all unambiguous letters allowed in motifs.
         """
-        all_letters = self._ambiguity_info.keys()
-        all_letters.sort()
-
+        all_letters = sorted(self._ambiguity_info)
         unambig_letters = []
 
         for letter in all_letters:
@@ -154,6 +162,7 @@ class Schema:
 
 # -- Alphabets
 
+
 class SchemaDNAAlphabet(Alphabet.Alphabet):
     """Alphabet of a simple Schema for DNA sequences.
 
@@ -165,16 +174,17 @@ class SchemaDNAAlphabet(Alphabet.Alphabet):
     o * - Any letter
     """
     letters = ["G", "A", "T", "C", "*"]
-    
-    alphabet_matches = {"G" : "G",
-                        "A" : "A",
-                        "T" : "T",
-                        "C" : "C",
-                        "*" : "GATC"}
+
+    alphabet_matches = {"G": "G",
+                        "A": "A",
+                        "T": "T",
+                        "C": "C",
+                        "*": "GATC"}
 
 # -- GA schema finder
 
-class GeneticAlgorithmFinder:
+
+class GeneticAlgorithmFinder(object):
     """Find schemas using a genetic algorithm approach.
 
     This approach to finding schema uses Genetic Algorithms to evolve
@@ -185,7 +195,7 @@ class GeneticAlgorithmFinder:
     can be overridden easily by creating a GeneticAlgorithmFinder
     with a different alphabet.
     """
-    def __init__(self, alphabet = SchemaDNAAlphabet()):
+    def __init__(self, alphabet=SchemaDNAAlphabet()):
         """Initialize a finder to get schemas using Genetic Algorithms.
 
         Arguments:
@@ -213,9 +223,9 @@ class GeneticAlgorithmFinder:
         this function.
         """
         self.motif_generator = RandomMotifGenerator(self.alphabet)
-        
-        self.mutator = SinglePositionMutation(mutation_rate = 0.1)
-        self.crossover = SinglePointCrossover(crossover_prob = 0.25)
+
+        self.mutator = SinglePositionMutation(mutation_rate=0.1)
+        self.crossover = SinglePointCrossover(crossover_prob=0.25)
         self.repair = AmbiguousRepair(Schema(self.alphabet.alphabet_matches),
                                       4)
         self.base_selector = TournamentSelection(self.mutator, self.crossover,
@@ -250,20 +260,21 @@ class GeneticAlgorithmFinder:
             # convert the Genome from a MutableSeq to a Seq so that
             # the schemas are just strings (and not array("c")s)
             seq_genome = org.genome.toseq()
-            schema_info[seq_genome.data] = org.fitness
+            schema_info[str(seq_genome)] = org.fitness
 
         return PatternRepository(schema_info)
 
 # -- fitness classes
 
-class DifferentialSchemaFitness:
+
+class DifferentialSchemaFitness(object):
     """Calculate fitness for schemas that differentiate between sequences.
     """
     def __init__(self, positive_seqs, negative_seqs, schema_evaluator):
         """Initialize with different sequences to evaluate
 
         Arguments:
-        
+
         o positive_seq - A list of SeqRecord objects which are the 'positive'
         sequences -- the ones we want to select for.
 
@@ -291,20 +302,20 @@ class DifferentialSchemaFitness:
         """
         # convert the genome into a string
         seq_motif = genome.toseq()
-        motif = seq_motif.data
-        
+        motif = str(seq_motif)
+
         # get the counts in the positive examples
         num_pos = 0
         for seq_record in self._pos_seqs:
             cur_counts = self._schema_eval.num_matches(motif,
-                                                      seq_record.seq.data)
+                                                      str(seq_record.seq))
             num_pos += cur_counts
 
         # get the counts in the negative examples
         num_neg = 0
         for seq_record in self._neg_seqs:
             cur_counts = self._schema_eval.num_matches(motif,
-                                                      seq_record.seq.data)
+                                                      str(seq_record.seq))
 
             num_neg += cur_counts
 
@@ -318,11 +329,12 @@ class DifferentialSchemaFitness:
         motif_size = motif_size * 4.0
 
         discerning_power = num_pos - num_neg
-        
+
         diff = (discerning_power * motif_size) / float(num_ambiguous)
         return diff
 
-class MostCountSchemaFitness:
+
+class MostCountSchemaFitness(object):
     """Calculate a fitness giving weight to schemas that match many times.
 
     This fitness function tries to maximize schemas which are found many
@@ -332,7 +344,7 @@ class MostCountSchemaFitness:
         """Initialize with sequences to evaluate.
 
         Arguments:
-        
+
         o seq_records -- A set of SeqRecord objects which we use to
         calculate the fitness.
 
@@ -351,22 +363,23 @@ class MostCountSchemaFitness:
         """
         # convert the genome into a string
         seq_motif = genome.toseq()
-        motif = seq_motif.data
-        
+        motif = str(seq_motif)
+
         # find the number of times the genome matches
         num_times = 0
         for seq_record in self._records:
             cur_counts = self._evaluator.num_matches(motif,
-                                                     seq_record.seq.data)
+                                                     str(seq_record.seq))
             num_times += cur_counts
 
         return num_times
 
+
 # -- Helper classes
-class RandomMotifGenerator:
+class RandomMotifGenerator(object):
     """Generate a random motif within given parameters.
     """
-    def __init__(self, alphabet, min_size = 12, max_size = 17):
+    def __init__(self, alphabet, min_size=12, max_size=17):
         """Initialize with the motif parameters.
 
         Arguments:
@@ -382,7 +395,7 @@ class RandomMotifGenerator:
 
     def random_motif(self):
         """Create a random motif within the given parameters.
-        
+
         This returns a single motif string with letters from the given
         alphabet. The size of the motif will be randomly chosen between
         max_size and min_size.
@@ -396,14 +409,15 @@ class RandomMotifGenerator:
 
         return MutableSeq(motif, self._alphabet)
 
-class SimpleFinisher:
+
+class SimpleFinisher(object):
     """Determine when we are done evolving motifs.
 
     This takes the very simple approach of halting evolution when the
     GA has proceeded for a specified number of generations and has
     a given number of unique schema with positive fitness.
     """
-    def __init__(self, num_schemas, min_generations = 100):
+    def __init__(self, num_schemas, min_generations=100):
         """Initialize the finisher with its parameters.
 
         Arguments:
@@ -438,7 +452,8 @@ class SimpleFinisher:
         return 0
 # ---
 
-class SchemaFinder:
+
+class SchemaFinder(object):
     """Find schema in a set of sequences using a genetic algorithm approach.
 
     Finding good schemas is very difficult because it takes forever to
@@ -450,8 +465,8 @@ class SchemaFinder:
     in a set of DNA sequences, but the finder can be customized to deal
     with any type of data.
     """
-    def __init__(self, num_schemas = 100,
-                 schema_finder = GeneticAlgorithmFinder()):
+    def __init__(self, num_schemas=100,
+                 schema_finder=GeneticAlgorithmFinder()):
         self.num_schemas = num_schemas
         self._finder = schema_finder
 
@@ -476,7 +491,8 @@ class SchemaFinder:
         return self._finder.find_schemas(fitness_evaluator.calculate_fitness,
                                          self.num_schemas)
 
-class SchemaCoder:
+
+class SchemaCoder(object):
     """Convert a sequence into a representation of ambiguous motifs (schemas).
 
     This takes a sequence, and returns the number of times specified
@@ -512,7 +528,7 @@ class SchemaCoder:
         schema_counts = []
 
         for schema in self._schemas:
-            num_counts = self._converter.num_matches(schema, sequence.data)
+            num_counts = self._converter.num_matches(schema, str(sequence))
             schema_counts.append(num_counts)
 
         # normalize the counts to go between zero and one
@@ -528,7 +544,8 @@ class SchemaCoder:
 
         return schema_counts
 
-def matches_schema(pattern, schema, ambiguity_character = '*'):
+
+def matches_schema(pattern, schema, ambiguity_character='*'):
     """Determine whether or not the given pattern matches the schema.
 
     Arguments:
@@ -547,18 +564,19 @@ def matches_schema(pattern, schema, ambiguity_character = '*'):
     # check each position, and return a non match if the schema and pattern
     # are non ambiguous and don't match
     for pos in range(len(pattern)):
-        if (schema[pos] != ambiguity_character and
-            pattern[pos] != ambiguity_character and
-            pattern[pos] != schema[pos]):
-            
+        if schema[pos] != ambiguity_character and \
+           pattern[pos] != ambiguity_character and \
+           pattern[pos] != schema[pos]:
+
             return 0
 
     return 1
 
-class SchemaFactory:
+
+class SchemaFactory(object):
     """Generate Schema from inputs of Motifs or Signatures.
     """
-    def __init__(self, ambiguity_symbol = '*'):
+    def __init__(self, ambiguity_symbol='*'):
         """Initialize the SchemaFactory
 
         Arguments:
@@ -595,9 +613,8 @@ class SchemaFactory:
         matched_count = 0
         assert total_count > 0, "Expected to have motifs to match"
         while (float(matched_count) / float(total_count)) < motif_percent:
-            
             new_schema, matching_motifs = \
-                        self._get_unique_schema(schema_info.keys(),
+                        self._get_unique_schema(list(schema_info.keys()),
                                                 all_motifs, num_ambiguous)
 
             # get the number of counts for the new schema and clean up
@@ -611,7 +628,6 @@ class SchemaFactory:
                 # represented by this schema
                 all_motifs.remove(motif)
 
-
             # all the schema info
             schema_info[new_schema] = schema_counts
 
@@ -642,13 +658,13 @@ class SchemaFactory:
         # we'll keep doing this until we get a completely new schema that
         # doesn't match any old schema
         num_tries = 0
-        
-        while 1:
+
+        while True:
             # pick a motif to work from and make a schema from it
             cur_motif = random.choice(motif_list)
-            
+
             num_tries += 1
-                
+
             new_schema, matching_motifs = \
                         self._schema_from_motif(cur_motif, motif_list,
                                                 num_ambiguous)
@@ -697,8 +713,8 @@ class SchemaFactory:
         new_schema_list = list(motif)
         for add_ambiguous in range(num_ambiguous):
             # add an ambiguous position in a new place in the motif
-            while 1:
-                ambig_pos = random.choice(range(len(new_schema_list)))
+            while True:
+                ambig_pos = random.choice(list(range(len(new_schema_list))))
 
                 # only add a position if it isn't already ambiguous
                 # otherwise, we'll try again
@@ -716,6 +732,6 @@ class SchemaFactory:
                 matched_motifs.append(motif)
 
         return new_schema, matched_motifs
-            
+
     def from_signatures(self, signature_repository, num_ambiguous):
         raise NotImplementedError("Still need to code this.")
